clean_data_used = False
wrt_raw_data_used = False
run_experiment_used = False
run_experiment_encoder = None
run_experiment_target_encoder = None
run_experiment_target_encoder_used = False
run_experiment_encoder_used = False
#below global variables are related to warm_start feature of auto_ml. Please don't change them. 
run_experiment_warm_start = False
run_experiment_x_train = None
run_experiment_x_test = None
run_experiment_y_train = None
run_experiment_y_test = None
run_experiment_model = None
run_experiment_datatransformed = None
run_experiment_labeldata = None